import { autocompletion } from '@codemirror/autocomplete';
import { defaultKeymap, history, historyKeymap, indentWithTab, insertNewlineKeepIndent, insertTab, } from '@codemirror/commands';
import { syntaxHighlighting } from '@codemirror/language';
import { EditorView, keymap, placeholder } from '@codemirror/view';
import { ActionName } from '../../bundle/config/action-names';
import { logger } from '../../logger';
import { Action as A, formatter as f } from '../../shortcuts';
import { insertLink, toH1, toH2, toH3, toH4, toH5, toH6, toggleBold, toggleItalic, toggleStrikethrough, toggleUnderline, wrapToCodeBlock, wrapToInlineCode, wrapToYfmCut, wrapToYfmNote, } from '../commands';
import { FileUploadHandlerFacet } from './files-upload-facet';
import { gravityHighlightStyle, gravityTheme } from './gravity';
import { PairingCharactersExtension } from './pairing-chars';
import { ReactRendererFacet } from './react-facet';
import { SearchPanelPlugin } from './search-plugin/plugin';
import { yfmLang } from './yfm';
export function createCodemirror(params) {
    const { doc, reactRenderer, onCancel, onScroll, onSubmit, onChange, onDocChange, disabledExtensions = {}, keymaps = [], receiver, yfmLangOptions, extensions: extraExtensions, placeholder: placeholderContent, autocompletion: autocompletionConfig, } = params;
    const extensions = [gravityTheme, placeholder(placeholderContent)];
    if (!disabledExtensions.history) {
        extensions.push(history());
    }
    extensions.push(syntaxHighlighting(gravityHighlightStyle), keymap.of([
        { key: f.toCM(A.Bold), run: withLogger(ActionName.bold, toggleBold) },
        { key: f.toCM(A.Italic), run: withLogger(ActionName.italic, toggleItalic) },
        { key: f.toCM(A.Strike), run: withLogger(ActionName.strike, toggleStrikethrough) },
        { key: f.toCM(A.Underline), run: withLogger(ActionName.underline, toggleUnderline) },
        { key: f.toCM(A.Link), run: withLogger(ActionName.link, insertLink) },
        { key: f.toCM(A.Heading1), run: withLogger(ActionName.heading1, toH1) },
        { key: f.toCM(A.Heading2), run: withLogger(ActionName.heading2, toH2) },
        { key: f.toCM(A.Heading3), run: withLogger(ActionName.heading3, toH3) },
        { key: f.toCM(A.Heading4), run: withLogger(ActionName.heading4, toH4) },
        { key: f.toCM(A.Heading5), run: withLogger(ActionName.heading5, toH5) },
        { key: f.toCM(A.Heading6), run: withLogger(ActionName.heading6, toH6) },
        { key: f.toCM(A.Code), run: withLogger(ActionName.code_inline, wrapToInlineCode) },
        { key: f.toCM(A.CodeBlock), run: withLogger(ActionName.code_block, wrapToCodeBlock) },
        { key: f.toCM(A.Cut), run: withLogger(ActionName.yfm_cut, wrapToYfmCut) },
        { key: f.toCM(A.Note), run: withLogger(ActionName.yfm_note, wrapToYfmNote) },
        {
            key: f.toCM(A.Cancel),
            preventDefault: true,
            run: () => {
                onCancel();
                return true;
            },
        },
        {
            key: f.toCM(A.Submit),
            preventDefault: true,
            run: () => {
                onSubmit();
                return true;
            },
        },
        { key: 'Tab', preventDefault: true, run: insertTab },
        {
            key: 'Enter',
            shift: insertNewlineKeepIndent,
        },
        indentWithTab,
        ...defaultKeymap,
        ...(disabledExtensions.history ? [] : historyKeymap),
        ...keymaps,
    ]), autocompletion(autocompletionConfig), yfmLang(yfmLangOptions), ReactRendererFacet.of(reactRenderer), PairingCharactersExtension, EditorView.lineWrapping, EditorView.contentAttributes.of({ spellcheck: 'true' }), EditorView.domEventHandlers({
        scroll(event) {
            onScroll(event);
        },
    }), SearchPanelPlugin({
        anchorSelector: '.g-md-search-anchor',
        receiver,
    }));
    if (params.uploadHandler) {
        extensions.push(FileUploadHandlerFacet.of({
            fn: params.uploadHandler,
            imgWithDimms: params.needImgDimms,
        }));
    }
    if (extraExtensions) {
        extensions.push(...extraExtensions);
    }
    return new EditorView({
        doc,
        extensions,
        dispatchTransactions: (trs, view) => {
            view.update(trs);
            onChange();
            if (trs.some((tr) => tr.docChanged)) {
                onDocChange();
            }
        },
    });
}
export function withLogger(action, command) {
    return (...args) => {
        logger.action({ mode: 'markup', source: 'keymap', action });
        return command(...args);
    };
}
